#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <cfloat>

#include <sstream>

#include "ida.h"
#include "wrap_ida.h"


void IDA::set_options(double rtol, double atol)
{
  rel_tol = rtol;
  abs_tol = atol;	
}

ColumnVector IDA::do_integrate (double tout)
{

  ColumnVector retval;
  N_Vector abstol;
  double tret;
//WRITE THE IDA FUNCTION CALL TO INTEGRATE TO TIME tout and return solution vector

  if (!initialized)
  {
    initialized = true;
    //Initialize once
    mem = IDACreate();
    errchk = IDAInit(mem, f, tm, s, derivs);
    if (errchk != 0)
    {
      //Catch error
    }
  
    rel_tol = 1e-4;
    abstol = 1e-8;

    errchk = IDASVtolerances(mem, rtol, *abstol);
    if (errchk != 0)
    {
      //Catch error
    }
 
    errchk = IDADense(mem, 3);
    if (errchk != 0)
    {
      //Catch error
    } 
  }

  errchk = IDASolve(mem, tout, &tret, s, derivs, IDA_NORMAL); 
  if (errchk != 0)
  {
    //Catch error
  }

  return s;

}

Matrix IDA::integrate (const ColumnVector& tout, Matrix& xdot_out)
{
  Matrix retval;

  octave_idx_type n_out = tout.capacity ();
  octave_idx_type n = size ();

  if (n_out > 0 && n > 0)
    {
      retval.resize (n_out, n);
      xdot_out.resize (n_out, n);

      for (octave_idx_type i = 0; i < n; i++)
        {
          retval.elem (0, i) = x.elem (i);
          xdot_out.elem (0, i) = xdot.elem (i);
        }

      for (octave_idx_type j = 1; j < n_out; j++)
        {
          ColumnVector x_next = do_integrate (tout.elem (j));

          for (octave_idx_type i = 0; i < n; i++)
            {
              retval.elem (j, i) = x_next.elem (i);
              xdot_out.elem (j, i) = xdot.elem (i);
            }
        }
    }

  return retval;
}

